@@ -1,12 +1,16 @@
 import asyncio
+import os
 import threading
-from asyncio import Task, Future
-from typing import Tuple, Dict, Any, Set, Optional
+from asyncio import AbstractEventLoop, Task, Future
+from typing import Tuple, Dict, Any, Set, Optional, Coroutine
 
 
 class Singleton(type):
     def __init__(
-        cls, name: str, bases: Tuple[type], namespace: Dict[str, Any],
+        cls,
+        name: str,
+        bases: Tuple[type],
+        namespace: Dict[str, Any],
     ) -> None:
         cls.instance = None
         super().__init__(name, bases, namespace)
@@ -17,11 +21,11 @@ class Singleton(type):
         return cls.instance
 
 
-def onlyfirst(*coros, loop=None) -> Future:
+def onlyfirst(*coros: Coroutine, loop: Optional[AbstractEventLoop] = None) -> Future:
     """
     Execute multiple coroutines concurrently, returning only the results of the first execution.
 
-    When one is completed, the execution of other coroutines will be canceled.
+    When one is completed, the execution of other coroutines will be canceled.
     """
     loop = loop or asyncio.get_running_loop()
     tasks: Set[Task] = set()
@@ -97,3 +101,50 @@ class State(dict):
 
     def __delattr__(self, name: Any) -> None:
         del self[name]
+
+
+if os.name == "nt":
+    import winreg
+
+    def set_proxy(enable: bool, proxy: str) -> None:
+        """
+        设定系统的网络代理
+        """
+        key = winreg.OpenKey(
+            winreg.HKEY_CURRENT_USER,
+            "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
+            0,
+            winreg.KEY_WRITE,
+        )
+        winreg.SetValueEx(key, "ProxyEnable", 0, winreg.REG_DWORD, int(enable))
+        winreg.SetValueEx(key, "ProxyServer", 0, winreg.REG_SZ, proxy)
+        winreg.CloseKey(key)
+
+    def get_proxy() -> Tuple[bool, str]:
+        """
+        获取系统的网络代理设置
+        """
+        key = winreg.OpenKey(
+            winreg.HKEY_CURRENT_USER,
+            "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
+            0,
+            winreg.KEY_READ,
+        )
+        try:
+            return (
+                bool(winreg.QueryValueEx(key, "ProxyEnable")[0]),
+                winreg.QueryValueEx(key, "ProxyServer")[0],
+            )
+        except FileNotFoundError:
+            return False, ""
+        finally:
+            winreg.CloseKey(key)
+
+
+else:
+
+    def set_proxy(enable: bool, proxy: str) -> None:
+        pass
+
+    def get_proxy() -> Tuple[bool, str]:
+        return True, ""
